/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Cookie editor.
 *
 * ************************************************************************
 */

"use strict";

const NUM_SEC_PER_DAY         = 86400; // 24*60*60

var gCookieEditor = {
  _bundle      : null,
  _cookie      : null,

  _tbName      : null,
  _tbValue     : null,
  _tbHost      : null,
  _tbPath      : null,
  _cbHttpOnly  : null,
  _dpExpire    : null,
  _tpExpire    : null,
  _Yes         : "Yes",
  _No          : "No",

  init: function() {
    this._cookie = window.arguments.length == 0 ? null : window.arguments[0];
    // Check argument
    if (this._cookie == null || typeof(this._cookie.host) == "undefined" || this._cookie.getType()  != COOKIE_TYPE_STANDARD) {
      window.close();
      return;
    }
    this._bundle     = document.getElementById("bundleCookieKeeper");
    this._Yes        = this._bundle.getString("Yes");
    this._No         = this._bundle.getString("No");

    this._tbName     = document.getElementById("tbName");
    this._tbValue    = document.getElementById("tbValue");
    this._tbHost     = document.getElementById("tbHost");
    this._tbPath     = document.getElementById("tbPath");
    this._cbHttpOnly = document.getElementById("cbHttpOnly");
    this._dpExpire   = document.getElementById("dpExpires");
    this._tpExpire   = document.getElementById("tpExpires");

    var radioGroupSecure = document.getElementById("radioGroupSecure");
    var radioGroupExpire = document.getElementById("radioGroupExpire");

    this._tbName.value       = this._cookie.name;
    this._tbValue.value      = this._cookie.value;
    this._tbHost.value       = this._cookie.host;
    this._tbPath.value       = this._cookie.path;

    if (this._cookie.isSecure) {
      radioGroupSecure.selectedItem = document.getElementById("radioSecureYes");
    } else {
      radioGroupSecure.selectedItem = document.getElementById("radioSecureNo");
    }

    if (this._cookie.isHttpOnly) {
      this._cbHttpOnly.checked = true;
      this._cbHttpOnly.label   = this._Yes;
    } else {
      this._cbHttpOnly.checked = false;
      this._cbHttpOnly.label   = this._No;
    }
    this._cbHttpOnly.addEventListener("click", this.onHTTPBoxChange, false);

    var eDate = null;
    if (this._cookie.isSession) {
      eDate = new Date(this._cookie.expiry);
      // Date widget limit:
      if (eDate.getFullYear() > 9999) {
        eDate.setFullYear(9999);
      }
      this._dpExpire.disabled = true;
      this._tpExpire.disabled = true;
      radioGroupExpire.selectedItem = document.getElementById("radioExpireSession");
    } else {
      eDate = new Date(this._cookie.expires);
      radioGroupExpire.selectedItem = document.getElementById("radioExpireDate");
    }

    this._dpExpire.value = "" + eDate.getFullYear() + "-" + (eDate.getMonth() + 1) + "-" + eDate.getDate();
    this._tpExpire.value = "" + eDate.getHours()    + ":" + eDate.getMinutes()     + ":" + eDate.getSeconds();

    this._tbHost.focus();
  },

  _getExpireValue: function() {
    var dExp = this._dpExpire.dateValue;
    var tExp = this._tpExpire.dateValue;
    return Math.floor(dExp.valueOf()/1000) + (tExp.getHours()*60+tExp.getMinutes())*60 + tExp.getSeconds();
  },

  onSave: function() {
    var cm = Cc["@mozilla.org/cookiemanager;1"].getService(Ci.nsICookieManager2);

    var newExpireValue = this._getExpireValue();
    if (newExpireValue < Math.round(Date.now()/1000)) {
      var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
      var response = promptService.confirm(window,
        this._bundle.getString("cookiekeeper.label"),
        this._bundle.getString("confirm_edit_cookie_expires"));
      if (!response) {
        return;
      }
    }

    // Is cookie primary key change ?
    if (this._cookie.host != this._tbHost.value ||
        this._cookie.name != this._tbName.value ||
        this._cookie.path != this._tbPath.value) {
      cm.remove(this._cookie.host, this._cookie.name, this._cookie.path, false);
    }

    var radioGroupSecure = document.getElementById("radioGroupSecure");
    var isSecure = radioGroupSecure.selectedIndex == 1;

    cm.add(this._tbHost.value, this._tbPath.value, this._tbName.value, this._tbValue.value,
           isSecure, this._cbHttpOnly.checked, this._cookie.isSession, newExpireValue);

    window.close();
  },

  uninit: function() {
    this._cookie = null;
  },

  onWindowKeyPress: function (aEvent) {
    if (aEvent.keyCode == KeyEvent.DOM_VK_ESCAPE) {
      window.close();
    }
  },

  onRadioExpire: function(aValue) {
    if (aValue == "date") {
      this._dpExpire.disabled = false;
      this._tpExpire.disabled = false;
      this._cookie.isSession  = false;

      var dNow = Date.now()/1000;
      if (this._getExpireValue() < dNow) {
        var eDate = new Date((dNow + NUM_SEC_PER_DAY) * 1000);

        this._dpExpire.value = "" + eDate.getFullYear() + "-" + (eDate.getMonth() + 1) + "-" + eDate.getDate();
        this._tpExpire.value = "" + eDate.getHours()    + ":" + eDate.getMinutes()     + ":" + eDate.getSeconds();
      }

    } else { // if (aValue == "session") {
      this._dpExpire.disabled = true;
      this._tpExpire.disabled = true;
      this._cookie.isSession  = true;
    }
  },

  onHTTPBoxChange: function() {
    if (gCookieEditor._cbHttpOnly.checked) {
      gCookieEditor._cbHttpOnly.label = gCookieEditor._Yes;
    } else {
      gCookieEditor._cbHttpOnly.label = gCookieEditor._No;
    }
  }
};
