/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Dom storage viewer.
 *
 * ************************************************************************
 */

"use strict";

const VALUE_SIZE_ASYNC_LOAD = 50*1024;

var gDomStorageEditor = {
  _item : null, // The DOM Storage item
  _json : null,

  _editMode     : false, // false to be in viewer mode, true, edit mode

  // UI items:
  _bundle       : null,
  _btnFormatJson: null,
  _btnPackJson  : null,
  _btnEdit      : null,
  _btnSave      : null,

  _deckHost     : null,
  _tbHostView   : null,
  _tbHostEdit   : null,
  _mlProtocol   : null,
  _mlPort       : null,

  _tbKey        : null,
  _tbContent    : null,
  _labelSize    : null,
  _tbJsonError  : null,

  init: function() {
    if (window.arguments == null) {
      window.close();
      return;
    }
    this._item = window.arguments.length >= 1 ? window.arguments[0] : null;
    if (this._item == null || this._item.getType() != COOKIE_TYPE_STORAGE) {
      window.close();
      return;
    }
    this._editMode = window.arguments.length >= 2 ? window.arguments[1] : false;

    this._bundle        = document.getElementById("bundleCookieKeeper");
    this._btnFormatJson = document.getElementById("btnFormatJson");
    this._btnPackJson   = document.getElementById("btnPackJson");
    this._btnEdit       = document.getElementById("btnEdit");
    this._btnSave       = document.getElementById("btnSave");
    this._deckHost      = document.getElementById("deckHost");
    this._tbHostView    = document.getElementById("tbHostView");
    this._tbHostEdit    = document.getElementById("tbHostEdit");
    this._mlProtocol    = document.getElementById("mlProtocol");
    this._mlPort        = document.getElementById("mlPort");
    this._tbKey         = document.getElementById("tbKey");
    this._tbContent     = document.getElementById("tbContent");
    this._labelSize     = document.getElementById("labelSize");
    this._tbJsonError   = document.getElementById("tbJsonError");

    this._btnFormatJson.parentNode.selectedPanel = this._btnFormatJson;

    if (this._editMode) {
      this._initEditor(true);
    } else {
      this._initViewer(true);
    }

  },

  /// Init this window in a viewer mode
  _initViewer: function(aInitialLoad) {
    if (aInitialLoad) {
      window.document.title     = this._bundle.getString("dom_storage.viewer.title");
      this._btnEdit.hidden      = false;
      this._btnSave.hidden      = true;
      this._tbContent.readOnly  = true;
      this._tbHostView.readOnly = true;
      this._tbKey.readOnly      = true;

      this._deckHost.selectedPanel = document.getElementById("hbHostView");
      this._tbHostView.value    = this._item.host;
      this._tbKey.value         = this._item.key;
    }

    if (this._item.value == null) {
      this._tbContent.disabled     = true;
      this._tbContent.value        = this._bundle.getString("value.null");
      this._btnFormatJson.disabled = true;
      this._btnPackJson.disabled   = true;
    } else if (aInitialLoad && this._item.value.length >= VALUE_SIZE_ASYNC_LOAD) {
      // If the text content is big, display the window first
      // Then set the value
      // In some cases, it may take a few seconds to display, and this freeze Mozilla
      this._btnFormatJson.disabled = true;
      this._btnPackJson.disabled   = true;
      window.setTimeout(function() {gDomStorageEditor._initViewer(false);}, 50);
    } else {
      try {
        this._json = JSON.parse(this._item.value);
        this._btnFormatJson.disabled = false;
        this._btnPackJson.disabled   = false;
      } catch (ex) {
        this._json = null;
        this._btnFormatJson.disabled = true;
        this._btnPackJson.disabled   = true;
      }
      this._tbContent.value = this._item.value;
      this._labelSize.value = gCommon.getHumanSize(this._item.value.length, this._bundle);
    }
  },

  /// Init this window as an editor
  _initEditor: function(aInitialLoad) {
    if (aInitialLoad) {
      window.document.title    = this._bundle.getString("dom_storage.editor.title");
      this._btnEdit.hidden     = true;
      this._btnSave.hidden     = false;
      this._tbContent.readOnly = false;
      this._tbKey.readOnly     = false;

      this._tbHostEdit.value   = this._item.host;
      this._mlProtocol.value   = this._item.proto;
      this._mlPort.value       = this._item.port;
      this._tbKey.value        = this._item.key;
      this._tbContent.value    = this._item.value;
      this._labelSize.value    = gCommon.getHumanSize(this._item.value.length, this._bundle);
      this._deckHost.selectedPanel = document.getElementById("hbHostEdit");
      this._btnPackJson.parentNode.selectedPanel = this._btnFormatJson;

      this._tbContent.focus();
    }

    this._json = null;
  },

  uninit: function() {
    this._item          = null;
    this._json          = null;

    this._bundle        = null;
    this._btnFormatJson = null;
    this._btnPackJson   = null;
    this._btnEdit       = null;
    this._btnSave       = null;
    this._deckHost      = null;
    this._tbHostView    = null;
    this._tbHostEdit    = null;
    this._mlProtocol    = null;
    this._mlPort        = null;
    this._tbContent     = null;
    this._tbKey         = null;
    this._labelSize     = null;
    this._tbJsonError   = null;
  },

  onFormatJson: function() {
    try {
      if (this._editMode) {
        this._json = JSON.parse(this._tbContent.value);
      }
      let jsonText = JSON.stringify(this._json, null, 2);
      this._tbContent.value = jsonText;
      this._labelSize.value = gCommon.getHumanSize(jsonText.length, this._bundle);
      this._btnFormatJson.parentNode.selectedPanel = this._btnPackJson;
      this._tbJsonError.value = "";
    } catch (ex) {
      this._tbJsonError.value = ex.toString();
      log.e(ex);
    }
    this._tbContent.focus();
  },

  onPackJson: function() {
    try {
      if (this._editMode) {
        this._json = JSON.parse(this._tbContent.value);
      }
      let jsonText = JSON.stringify(this._json, null, 0);
      this._tbContent.value = jsonText;
      this._labelSize.value = gCommon.getHumanSize(jsonText.length, this._bundle);
      this._btnPackJson.parentNode.selectedPanel = this._btnFormatJson;
      this._tbJsonError.value = "";
    } catch (ex) {
      this._tbJsonError.value = ex.toString();
      log.e(ex);
    }
    this._tbContent.focus();
  },

  onEdit: function() {
    this._editMode = true;
    this._tbHostView.value = "";
    this._initEditor(true);
  },

  onSave: function() {
    // Make scope:
    let host     = this._tbHostEdit.value;
    let protocol = this._mlProtocol.value;
    let port     = this._mlPort.value;
    let key      = this._tbKey.value;
    let value    = this._tbContent.value;

    if (host.length == 0) {
      alert("Host not set"); // FIXME
      return;
    }
    if (key.length == 0) {
      alert("Key empty"); // FIXME
      return;
    }
    let scope = gCommon.reverseStr(host);
    scope = scope + (protocol.length > 0 ? (":" + protocol) : "");
    scope = scope + (port.length > 0 ? (":" + port) : "");

    let pKeyEquals = ((scope == this._item.scope) && (key == this._item.key));

    if (pKeyEquals && value == this._item.value) {
      // No change
    } else if (pKeyEquals) {
      // Just update the value
      this._item.value = value;
      gDomStorage.performInsertion([this._item]);
    } else {
      // New item, delete the old first
      gDomStorage.performDeletion([this._item], false);
      let item = gDomStorage.makeDomStorageObject(scope, key, value, null, null);
      gDomStorage.performInsertion([item]);
    }

    window.close();
  },

  onWindowKeyPress: function (aEvent) {
    if (aEvent.keyCode == KeyEvent.DOM_VK_ESCAPE) {
      window.close();
    }
  }

};
