/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Test Mozilla preferences.
 *
 * ************************************************************************
 */

"use strict";

var gTestMozPrefs = {

  doTest: function() {
    var pService      = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
    var defaultBranch = pService.QueryInterface(Ci.nsIPrefBranch2).getDefaultBranch("");
    var currentBranch = pService.getBranch("");
    var showWarn      = true;
    var appID         = Services.appinfo.ID;
    var appPrefInfos  = APP_MOZ_CONST[appID];

    if (typeof(appPrefInfos) != "object") {
      log.e("Unknown Mozilla based application");
      return;
    }

    // Get preference:
    if (currentBranch.getPrefType(PREF_WARN_MOZ) == currentBranch.PREF_BOOL) {
      showWarn = currentBranch.getBoolPref(PREF_WARN_MOZ);
    } else if (defaultBranch.getPrefType(PREF_WARN_MOZ) == defaultBranch.PREF_BOOL) {
      // Get default prefs:
      showWarn = defaultBranch.getBoolPref(PREF_WARN_MOZ);
    }

    if (showWarn) {
      if (appPrefInfos.NAME == FIREFOX || appPrefInfos.NAME == PALEMOON || appPrefInfos.NAME == KMELEON) {
        showWarn = this._firefox(defaultBranch, currentBranch, appID);
      } else if (appPrefInfos.NAME == SEAMONKEY) {
        showWarn = this._seamonkey(defaultBranch, currentBranch, appID);
      } else if (appPrefInfos.NAME == THUNDERBIRD) {
        showWarn = this._thunderbird(defaultBranch, currentBranch, appID);
      }

      if (showWarn) {
        let box = document.getElementById("notifyBrowserPref");
        box.hidden = false;
      }
    }

    if (appPrefInfos.NAME == THUNDERBIRD) {
      // Hide menuitemProtectBookmarkedCookies since there is no bookmark with Thunderbird
      let menuitem = document.getElementById("menuitemProtectBookmarkedCookies");
      menuitem.hidden = true;
      menuitem = document.getElementById("menuseparatorProtectBookmarkedCookies");
      menuitem.hidden = true;
    }
  },

  onShowPrefs: function(aEvent, aParam) {
    aEvent.stopPropagation();
    if (aParam == CK_PREFPANE_BROWSER) {
      let box = document.getElementById("notifyBrowserPref");
      box.close();
    }

    CookieKeeper.showPreferencesDialogue(window, aParam);
    document.getElementById("textBoxSearch").focus();
  },

  // -------------------------------------
  // Mozilla preferences alert:
  _firefox: function(defaultBranch, currentBranch, appID) {
    const PRIVATEBROWSING           = APP_MOZ_CONST[appID].PRIVATEBROWSING;
    const SANITIZE_ON_SHUTDOWN      = APP_MOZ_CONST[appID].SANITIZE_ON_SHUTDOWN;
    const COOKIES_CLEAR_ON_SHUTDOWN = APP_MOZ_CONST[appID].COOKIES_CLEAR_ON_SHUTDOWN;
    const COOKIES_BEHAVIOR          = APP_MOZ_CONST[appID].COOKIES_BEHAVIOR;
    const COOKIES_LIFETIME          = APP_MOZ_CONST[appID].COOKIES_LIFETIME;
    // const COOKIES_LIFETIME_DAYS  = APP_MOZ_CONST[appID].COOKIES_LIFETIME_DAYS;

    // Mozilla Firefox 24.0 default preferences:
    var privateBrowsing = false;
    var doSanitize      = false;
    var clearOnShutdown = true;
    var acceptCookies   = true;
    var changeLifetime  = false;
    var cookieBehavior  = 0;

    if (currentBranch.getPrefType(PRIVATEBROWSING) == currentBranch.PREF_BOOL) {
      // user preference:
      privateBrowsing = currentBranch.getBoolPref(PRIVATEBROWSING);
    } else if (defaultBranch.getPrefType(PRIVATEBROWSING) == defaultBranch.PREF_BOOL) {
      // default value:
      privateBrowsing = defaultBranch.getBoolPref(PRIVATEBROWSING);
    }

    if (currentBranch.getPrefType(SANITIZE_ON_SHUTDOWN) == currentBranch.PREF_BOOL) {
      doSanitize = currentBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    } else if (defaultBranch.getPrefType(SANITIZE_ON_SHUTDOWN) == defaultBranch.PREF_BOOL) {
      doSanitize = defaultBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    }

    if (doSanitize) {
      if (currentBranch.getPrefType(COOKIES_CLEAR_ON_SHUTDOWN) == currentBranch.PREF_BOOL) {
        clearOnShutdown = currentBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      } else if (defaultBranch.getPrefType(COOKIES_CLEAR_ON_SHUTDOWN) == defaultBranch.PREF_BOOL) {
        clearOnShutdown = defaultBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      }
    }

    if (currentBranch.getPrefType(COOKIES_BEHAVIOR) == currentBranch.PREF_INT) {
      cookieBehavior = currentBranch.getIntPref(COOKIES_BEHAVIOR);
    } else if (defaultBranch.getPrefType(COOKIES_BEHAVIOR) == defaultBranch.PREF_INT) {
      cookieBehavior = defaultBranch.getIntPref(COOKIES_BEHAVIOR);
    }

    acceptCookies = (cookieBehavior != MOZ_BEHAVIOR_DISABLED);


    if (currentBranch.getPrefType(COOKIES_LIFETIME) == currentBranch.PREF_INT) {
      let mozCookieLifetime = currentBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    } else if (defaultBranch.getPrefType(COOKIES_LIFETIME) == defaultBranch.PREF_INT) {
      let mozCookieLifetime = defaultBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }

    return (doSanitize && clearOnShutdown) || changeLifetime || privateBrowsing || !acceptCookies;
  },

  _seamonkey: function(defaultBranch, currentBranch, appID) {
    const SANITIZE_ON_SHUTDOWN      = APP_MOZ_CONST[appID].SANITIZE_ON_SHUTDOWN;
    const COOKIES_CLEAR_ON_SHUTDOWN = APP_MOZ_CONST[appID].COOKIES_CLEAR_ON_SHUTDOWN;
    const COOKIES_BEHAVIOR          = APP_MOZ_CONST[appID].COOKIES_BEHAVIOR;
    const COOKIES_LIFETIME          = APP_MOZ_CONST[appID].COOKIES_LIFETIME;

    // Mozilla SeaMonkey 2.21 default preferences:
    var doSanitize      = false;
    var clearOnShutdown = false;
    var acceptCookies   = true;
    var changeLifetime  = false;
    var cookieBehavior  = 0;

    if (currentBranch.getPrefType(SANITIZE_ON_SHUTDOWN) == currentBranch.PREF_BOOL) {
      doSanitize = currentBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    } else if (defaultBranch.getPrefType(SANITIZE_ON_SHUTDOWN) == defaultBranch.PREF_BOOL) {
      doSanitize = defaultBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    }

    if (doSanitize) {
      if (currentBranch.getPrefType(COOKIES_CLEAR_ON_SHUTDOWN) == currentBranch.PREF_BOOL) {
        clearOnShutdown = currentBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      } else if (defaultBranch.getPrefType(COOKIES_CLEAR_ON_SHUTDOWN) == defaultBranch.PREF_BOOL) {
        clearOnShutdown = defaultBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      }
    }

    if (currentBranch.getPrefType(COOKIES_BEHAVIOR) == currentBranch.PREF_INT) {
      cookieBehavior = currentBranch.getIntPref(COOKIES_BEHAVIOR);
    } else if (defaultBranch.getPrefType(COOKIES_BEHAVIOR) == defaultBranch.PREF_INT) {
      cookieBehavior = defaultBranch.getIntPref(COOKIES_BEHAVIOR);
    }

    acceptCookies = (cookieBehavior != MOZ_BEHAVIOR_DISABLED);

    if (currentBranch.getPrefType(COOKIES_LIFETIME) == currentBranch.PREF_INT) {
      let mozCookieLifetime = currentBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    } else if (defaultBranch.getPrefType(COOKIES_LIFETIME) == defaultBranch.PREF_INT) {
      let mozCookieLifetime = defaultBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }

    return (doSanitize && clearOnShutdown) || changeLifetime || !acceptCookies;
  },

  _thunderbird: function(defaultBranch, currentBranch, appID) {
    const COOKIES_BEHAVIOR = APP_MOZ_CONST[appID].COOKIES_BEHAVIOR;
    const COOKIES_LIFETIME = APP_MOZ_CONST[appID].COOKIES_LIFETIME;

    // Mozilla Thunderbird 24.0 default preferences:
    var acceptCookies  = true;
    var changeLifetime = false;
    var cookieBehavior = 0;

    if (currentBranch.getPrefType(COOKIES_BEHAVIOR) == currentBranch.PREF_INT) {
      cookieBehavior = currentBranch.getIntPref(COOKIES_BEHAVIOR);
    } else if (defaultBranch.getPrefType(COOKIES_BEHAVIOR) == defaultBranch.PREF_INT) {
      cookieBehavior = defaultBranch.getIntPref(COOKIES_BEHAVIOR);
    }

    acceptCookies = (cookieBehavior != MOZ_BEHAVIOR_DISABLED);

    if (currentBranch.getPrefType(COOKIES_LIFETIME) == currentBranch.PREF_INT) {
      let mozCookieLifetime = currentBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    } else if (defaultBranch.getPrefType(COOKIES_LIFETIME) == defaultBranch.PREF_INT) {
      let mozCookieLifetime = defaultBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }

    return changeLifetime || !acceptCookies;
  }
};
