/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Preferences for the browser tab.
 * Note: The checkbox values are changed with preferences.js
 * using the prefsObserver.
 *
 * ************************************************************************
 */

"use strict";

var gCookieKeeperBrowserHTMLPrefs = {
  init: function() {
    this.uninit();

    let cbPrivBrowsingAuto   = document.getElementById("browser_cb_PrivateBrowsingAutoStart");
    let cbSanitizeOnShutdown = document.getElementById("browser_cb_SanitizeOnShutdown");
    let formAcceptPolicy     = document.getElementById("browser_form_accept_policy");
    let formLifetimePolicy   = document.getElementById("browser_form_lifetime_policy");
    let hLifetimeDays        = document.getElementById("browser_input_lifetime_policy_days");

    cbPrivBrowsingAuto.addEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    cbSanitizeOnShutdown.addEventListener(EVENT_CUSTOM_CHANGED, this.onPermChange, false);
    formAcceptPolicy.addEventListener(EVENT_CUSTOM_CHANGED,     this.onPermChange, false);
    formLifetimePolicy.addEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    hLifetimeDays.addEventListener(EVENT_CUSTOM_CHANGED,        this.onLifetimeChanged, false);

    if (APP_MOZ_CONST[Services.appinfo.ID].NAME == FIREFOX) {
      this.initFirefox();
    } else if (APP_MOZ_CONST[Services.appinfo.ID].NAME == SEAMONKEY) {
      this.initSeamonkey();
    } else if (APP_MOZ_CONST[Services.appinfo.ID].NAME == THUNDERBIRD) {
      this.initThunderbird();
    } else if (APP_MOZ_CONST[Services.appinfo.ID].NAME == PALEMOON) {
      this.initFirefox();
    } else {
      log.e("Unknown application type: " + Services.appinfo.name);
      // Use firefox as default, even if it can failed.
      this.initFirefox();
    }

    this.onPermChange();
    this.onLifetimeChanged();
  },

  initFirefox: function() {
  },

  initSeamonkey: function() {
  },

  initThunderbird: function() {
    let hDiv = document.getElementById("browser_PrivateBrowsingAutoStart");
    hDiv.style.display = "none";
    hDiv = document.getElementById("browser_SanitizeOnShutdown");
    hDiv.style.display = "none";
  },

  uninit: function() {
    let cbPrivBrowsingAuto   = document.getElementById("browser_cb_PrivateBrowsingAutoStart");
    let cbSanitizeOnShutdown = document.getElementById("browser_cb_SanitizeOnShutdown");
    let formAcceptPolicy     = document.getElementById("browser_form_accept_policy");
    let formLifetimePolicy   = document.getElementById("browser_form_lifetime_policy");
    let hLifetimeDays        = document.getElementById("browser_input_lifetime_policy_days");

    cbPrivBrowsingAuto.removeEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    cbSanitizeOnShutdown.removeEventListener(EVENT_CUSTOM_CHANGED, this.onPermChange, false);
    formAcceptPolicy.removeEventListener(EVENT_CUSTOM_CHANGED,     this.onPermChange, false);
    formLifetimePolicy.removeEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    hLifetimeDays.removeEventListener(EVENT_CUSTOM_CHANGED,        this.onLifetimeChanged, false);
  },

  /**
   * @brief Update enbaled/disabled items, depending of the values.
   * @par
   * For example, if we always are in private browsing, all others options are meaningless
   * so disabled them.
   */
  onPermChange: function(evtChange) {
    let that                 = gCookieKeeperBrowserHTMLPrefs;
    let cbPrivBrowsingAuto   = document.getElementById("browser_cb_PrivateBrowsingAutoStart");
    let acceptPolicyValue    = document.getElementById("browser_form_accept_policy").value;
    let lifetimePolicyValue  = document.getElementById("browser_form_lifetime_policy").value;

    if (cbPrivBrowsingAuto.checked) {
      that.disableCheckBox("browser_cb_SanitizeOnShutdown",     true);
      that.disableCheckBox("browser_cb_ClearOnShutDownCookies", true);
      that.disableRadioGroup("browser_form_accept_policy",      true);
      that.disableRadioGroup("browser_form_lifetime_policy",    true);
      that.disableCheckBox("browser_cb_lifetime_policy_tpso",   true);
      document.getElementById("browser_input_lifetime_policy_days").setAttribute("disabled", "disabled");
      return;
    }

    that.disableCheckBox("browser_cb_SanitizeOnShutdown", false);
    if (document.getElementById("browser_cb_SanitizeOnShutdown").checked) {
      that.disableCheckBox("browser_cb_ClearOnShutDownCookies", false);
    } else {
      that.disableCheckBox("browser_cb_ClearOnShutDownCookies", true);
    }

    that.disableRadioGroup("browser_form_accept_policy", false);

    if (acceptPolicyValue == "2") {
      // "Reject all cookies" is selected, disable all lifetime policy elements
      that.disableRadioGroup("browser_form_lifetime_policy",  true);
      that.disableCheckBox("browser_cb_lifetime_policy_tpso", true);
      document.getElementById("browser_input_lifetime_policy_days").setAttribute("disabled", "disabled");
      return;
    }

    // Enable lifetime policy elements
    that.disableRadioGroup("browser_form_lifetime_policy",  false);
    that.disableCheckBox("browser_cb_lifetime_policy_tpso", false);

    if (lifetimePolicyValue == "3") {
      document.getElementById("browser_input_lifetime_policy_days").removeAttribute("disabled");
    } else {
      document.getElementById("browser_input_lifetime_policy_days").setAttribute("disabled", "disabled");
    }
  },

  onLifetimeChanged: function(evtChanged) {
    let value = document.getElementById("browser_input_lifetime_policy_days").value;
    let hElem = document.getElementById("browser_label_lifetime_policy_days");
    if (hElem.hasAttribute("data-l10n-dynid")) {
      let l10n   = hElem.getAttribute("data-l10n-dynid");
      let trText = gChrome.i18n.getMessage(l10n, [value]);
      hElem.textContent = trText;
    }
  },

  disableRadioGroup: function(name, disabled) {
    // Dirty way to disable radio input.
    // HTML is not very convenient for making UI :(
    let form = document.getElementById(name);
    if (form != null) {
      let nRadio = form.children.length;
      for (let i=0; i<nRadio; i++) {
        let hChild = form.children[i];
        if (disabled) {
          if (hChild.nodeName == "INPUT") {
            hChild.setAttribute("disabled", "disabled");
          } else if (hChild.nodeName == "LABEL") {
            hChild.classList.add("label_disabled");
          }
        } else {
          // enable the radio
          if (hChild.nodeName == "INPUT") {
            hChild.removeAttribute("disabled");
          } else if (hChild.nodeName == "LABEL") {
            hChild.classList.remove("label_disabled");
          }
        }
      }
    }
  },

  disableCheckBox: function(name, disabled) {
    let hInput = document.getElementById(name);
    if (hInput != null) {
      if (disabled) {
        hInput.setAttribute("disabled", "disabled");
        hInput.nextElementSibling.classList.add("label_disabled");
      } else {
        hInput.removeAttribute("disabled");
        hInput.nextElementSibling.classList.remove("label_disabled");
      }
    }
  }
};
