/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Mozilla Firefox preferences.
 *
 * ************************************************************************
 */

"use strict";

var gCookieKeeperFirefoxPrefs = {

  _cbPBAutoStart              : null, // check box privateBrowsingAutoStart
  _prefPBAutoStart            : null, // element preferences PREF_MOZ_PRIVATEBROWSING
  _imgPBAutoStart             : null,
  _cbSanitize                 : null,
  _prefSanitize               : null,
  _cbClearOnShutDownCookies   : null,
  _prefClearOnShutDownCookies : null,
  _imgClearOnShutDownCookies  : null,

  _groupAcceptPolicy          : null,
  _prefAcceptPolicy           : null,
  _imgWarnAcceptPolicy        : null,
  _groupLifetimePolicy        : null,
  _prefLifetimePolicy         : null,
  _imgWarnLifetimePolicy      : null,
  _hbLifetimePolicy           : null,
  _labelLifetimeDays          : null,
  _prefLifetimeDays           : null,
  _cbThirdpartySessionOnly    : null,

  _notShowWarn                : false,

  init: function() {
    var appinfos       = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo).QueryInterface(Ci.nsIXULRuntime);
    const appPrefInfos = APP_MOZ_CONST[appinfos.ID];

    // Get preference:
    if (Services.prefs.getPrefType(PREF_WARN_MOZ) == Services.prefs.PREF_BOOL) {
      this._notShowWarn = !Services.prefs.getBoolPref(PREF_WARN_MOZ);
    }

    this._cbPBAutoStart               = document.getElementById("privateBrowsingAutoStart");
    this._prefPBAutoStart             = document.getElementById(appPrefInfos.PRIVATEBROWSING);
    this._imgPBAutoStart              = document.getElementById("imgWarnPrivateBrowsingAutoStart");
    this._cbSanitize                  = document.getElementById("sanitizeOnShutdown");
    this._prefSanitize                = document.getElementById(appPrefInfos.SANITIZE_ON_SHUTDOWN);
    this._cbClearOnShutDownCookies    = document.getElementById("clearOnShutDownCookies");
    this._prefClearOnShutDownCookies  = document.getElementById(appPrefInfos.COOKIES_CLEAR_ON_SHUTDOWN);
    this._imgClearOnShutDownCookies   = document.getElementById("imgWarnClearOnShutDownCookies");
    this._groupAcceptPolicy           = document.getElementById("acceptPolicyRG");
    this._prefAcceptPolicy            = document.getElementById(appPrefInfos.COOKIES_BEHAVIOR);
    this._imgWarnAcceptPolicy         = document.getElementById("imgWarnAcceptNone");
    this._groupLifetimePolicy         = document.getElementById("lifetimePolicyRG");
    this._prefLifetimePolicy          = document.getElementById(appPrefInfos.COOKIES_LIFETIME);
    this._imgWarnLifetimePolicy       = document.getElementById("imgWarnPolicySession");
    this._tbLifetimeDays              = document.getElementById("tbLifetimeDays");
    this._prefLifetimeDays            = document.getElementById(appPrefInfos.COOKIES_LIFETIME_DAYS);
    this._labelLifetimeDays           = document.getElementById("labelLifetimePolicy");
    this._cbThirdpartySessionOnly     = document.getElementById("checkboxThirdpartySessionOnly");

    this.updateUI();
  },

  updateUI: function() {
    if (this._prefPBAutoStart.value) {
      this._imgPBAutoStart.hidden             = false;
      this._cbSanitize.disabled               = true;
      this._cbClearOnShutDownCookies.disabled = true;
      this._imgClearOnShutDownCookies.hidden  = true;
      this._groupAcceptPolicy.disabled        = true;
      this._groupLifetimePolicy.disabled      = true;
      this._tbLifetimeDays.disabled           = true;
      this._labelLifetimeDays.disabled        = true;
      this._cbThirdpartySessionOnly.disabled  = true;
      this._imgWarnAcceptPolicy.hidden        = true;
      this._imgWarnLifetimePolicy.hidden      = true;
    } else {
      this._groupAcceptPolicy.disabled  = false;
      this._imgPBAutoStart.hidden       = true;
      this._cbSanitize.disabled         = false;

      if (this._prefSanitize.value) {
        this._cbClearOnShutDownCookies.disabled = false;
        if (this._prefClearOnShutDownCookies.value) {
          this._imgClearOnShutDownCookies.hidden = false;
        } else {
          this._imgClearOnShutDownCookies.hidden = true;
        }
      } else {
        this._cbClearOnShutDownCookies.disabled = true;
        this._imgClearOnShutDownCookies.hidden  = true;
      }

      if (this._prefAcceptPolicy.value == MOZ_BEHAVIOR_DISABLED) {
        this._groupLifetimePolicy.disabled     = true;
        this._tbLifetimeDays.disabled          = true;
        this._labelLifetimeDays.disabled       = true;
        this._cbThirdpartySessionOnly.disabled = true;
        this._imgWarnAcceptPolicy.hidden       = this._notShowWarn;
        this._imgWarnLifetimePolicy.hidden     = true;
      } else {
        this._groupLifetimePolicy.disabled     = false;
        this._labelLifetimeDays.disabled       = false;
        this._cbThirdpartySessionOnly.disabled = false;
        this._imgWarnAcceptPolicy.hidden       = true;

        if (this._prefLifetimePolicy.value == MOZ_ACCEPT_FOR_N_DAYS) {
          this._tbLifetimeDays.disabled     = false;
        } else {
          this._tbLifetimeDays.disabled     = true;
        }
        if (this._prefLifetimePolicy.value == MOZ_ACCEPT_SESSION) {
          this._imgWarnLifetimePolicy.hidden = this._notShowWarn;
        } else {
          this._imgWarnLifetimePolicy.hidden = true;
        }
      }
    }
  }

};
