/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * CookieKeeper preferences panel.
 *
 * ************************************************************************
 */

 "use strict";

var gCookieKeeperPrefs = {

  init: function() {
    const PREF_FADE_IN = "browser.preferences.animateFadeIn";
    const pDefault     = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).QueryInterface(Ci.nsIPrefBranch2).getDefaultBranch("");
    // animateFadeIn is true on MacOSX by default, false on other OS.
    var fadeIn = false;


    if (Services.prefs.getPrefType(PREF_FADE_IN) == Services.prefs.PREF_BOOL) {
      // Get the user value if modified:
      fadeIn = Services.prefs.getBoolPref(PREF_FADE_IN);
    } else if (pDefault.getPrefType(PREF_FADE_IN) == pDefault.PREF_BOOL) {
      // Get the default value first, since it has not been changed
      fadeIn = pDefault.getBoolPref(PREF_FADE_IN);
    }

    if (!fadeIn) {
      // persist the pref window size, if animateFadeIn is not set
      // if animateFadeIn is true, the dialog may not be resize correctly with the persistance of width/height
      let prefsWindow = document.getElementById("cookieKeeperPrefsWindow");
      if (prefsWindow) {
        prefsWindow.setAttribute("persist", "lastSelected width height");
      }
    }
  },

  uninit: function() {
    try {
      if (typeof(gCookieKeeperProtItemsXUL) != "undefined") {
        gCookieKeeperProtItemsXUL.uninit();
      }
      if (typeof(gCookieKeeperPermissionsXUL) != "undefined") {
        // gCookieKeeperPermissionsXUL is undefined if we do not have
        // opened the permissions tabs at all.
        gCookieKeeperPermissionsXUL.uninit();
      }
    } catch (ex) {
      log.e(ex);
    }
  }
}
