/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Permissions panel.
 * Original code from browser/components/preferences/permissions.js (Mozilla Firefox 30.0a1 source code)
 *
 * ************************************************************************
 */

"use strict";

var gCookieKeeperPermissionsXUL = {
  _tree              : null,
  _oPermManager      : null,
  _sortColumn        : 1,
  _lastSelectedItems : null,

  _view: {
    _rowCount: 0,
    get rowCount() {
      let ap = gCookieKeeperPermissionsXUL._oPermManager.aPermissions;
      return ap.length;
    },

    getCellText: function(aRow, aColumn) {
      let ap = gCookieKeeperPermissionsXUL._oPermManager.aPermissions;
      if (aColumn.id == "treeColSite") {
        return ap[aRow].origin;
      } else if (aColumn.id == "treeColCapability") {
        return gCookieKeeperPermissionsXUL.getCapabilityString(ap[aRow].capability);
      }
      return "";
    },

    isSeparator: function(aIndex) { return false; },
    isSorted: function() { return false; },
    isContainer: function(aIndex) { return false; },
    setTree: function(aTree) {},
    getImageSrc: function(aRow, aColumn) {},
    getProgressMode: function(aRow, aColumn) {},
    getCellValue: function(aRow, aColumn) {},
    cycleHeader: function(aColumn) {},
    getRowProperties: function(aRow){ return ""; },
    getColumnProperties: function(aColumn) { return ""; },
    getCellProperties: function(aRow,aColumn) {
      if (aColumn.id == "treeColSite") {
        return "ltr";
      }
      return "";
    }
  },

  init: function() {
    this._bundle = document.getElementById("bundlePreferences");
    this._oPermManager = new CKPermissionsManager();
    this._oPermManager.fetchCookiesPermissions();
    this._oPermManager.addPermChangeObserver(this.onPermissionChange);

    this._tree = document.getElementById("treePermissions");
    let tbo    = this._tree.treeBoxObject;
    tbo.view   = this._view;

    this._sortColumn = 1;

    if (this._view.rowCount > 0) {
      let sel = this._tree.view.selection;
      this.onPermissionSort(null);

      sel.clearSelection();
      sel.select(0);
      tbo.invalidate();
      tbo.ensureRowIsVisible(0);

      document.getElementById("btnRemovePermission").disabled = false;
      document.getElementById("btnRemoveAllPermissions").disabled = false;
    } else {
      document.getElementById("btnRemovePermission").disabled = true;
      document.getElementById("btnRemoveAllPermissions").disabled = true;
    }

    // Add events listeners
    this._tree.addEventListener("select", this.onSelectionChange, false);
    this._tree.addEventListener("keypress", this.onPermissionKeyPress, false);

    let elem = document.getElementById("menu-popup-perms-context");
    elem.addEventListener("popupshowing", this.onShowContextMenu, false);
    elem = document.getElementById("mpcc-perm-allow");
    elem.addEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-perm-session");
    elem.addEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-perm-deny");
    elem.addEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-delete");
    elem.addEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-copy-domain");
    elem.addEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("perm-treechildren");
    elem.addEventListener("contextmenu", this.onContextMenu, false);
    elem = document.getElementById("textBoxUrl");
    elem.addEventListener("input", this.onTextboxURLInput, false);
    elem.addEventListener("keypress", this.onTextboxURLKeypress, false);
    elem.focus();

    elem = document.getElementById("btnBlock");
    elem.addEventListener("command", this.onAddPermission, false);
    elem = document.getElementById("btnSession");
    elem.addEventListener("command", this.onAddPermission, false);
    elem = document.getElementById("btnAllow");
    elem.addEventListener("command", this.onAddPermission, false);
    elem = document.getElementById("btnRemovePermission");
    elem.addEventListener("command", this.onDeletePermission, false);
    elem = document.getElementById("btnRemoveAllPermissions");
    elem.addEventListener("command", this.onDeleteAllPermissions, false);

    elem = document.getElementById("treeColSite");
    elem.addEventListener("click", this.onPermissionSort, false);
    elem = document.getElementById("treeColCapability");
    elem.addEventListener("click", this.onPermissionSort, false);
  },

  uninit: function() {
    if (this._oPermManager != null) {
      this._oPermManager.removePermChangeObserver(this.onPermissionChange);
      this._oPermManager.destructor();
      this._oPermManager = null;
    }
    if (this._tree != null) {
      this._tree.removeEventListener("select", this.onSelectionChange, false);
      this._tree.removeEventListener("keypress", this.onPermissionKeyPress, false);
      this._tree = null;
    }

    let elem = document.getElementById("menu-popup-perms-context");
    elem.removeEventListener("popupshowing", this.onShowContextMenu, false);
    elem = document.getElementById("mpcc-perm-allow");
    elem.removeEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-perm-session");
    elem.removeEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-perm-deny");
    elem.removeEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-delete");
    elem.removeEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("mpcc-copy-domain");
    elem.removeEventListener("command", this.onContextMenuItem, false);
    elem = document.getElementById("perm-treechildren");
    elem.removeEventListener("contextmenu", this.onContextMenu, false);

    elem = document.getElementById("textBoxUrl");
    elem.removeEventListener("input", this.onTextboxURLInput, false);
    elem.removeEventListener("keypress", this.onTextboxURLKeypress, false);

    elem = document.getElementById("btnBlock");
    elem.removeEventListener("command", this.onAddPermission, false);
    elem = document.getElementById("btnSession");
    elem.removeEventListener("command", this.onAddPermission, false);
    elem = document.getElementById("btnAllow");
    elem.removeEventListener("command", this.onAddPermission, false);
    elem = document.getElementById("btnRemovePermission");
    elem.removeEventListener("command", this.onDeletePermission, false);
    elem = document.getElementById("btnRemoveAllPermissions");
    elem.removeEventListener("command", this.onDeleteAllPermissions, false);

    elem = document.getElementById("treeColSite");
    elem.removeEventListener("click", this.onPermissionSort, false);
    elem = document.getElementById("treeColCapability");
    elem.removeEventListener("click", this.onPermissionSort, false);
  },

  getCapabilityString: function(aCapability) {
    var stringKey = null;
    switch (aCapability) {
    case MOZ_COOKIE_PERM_ALLOW:
      stringKey = "can";
      break;
    case MOZ_COOKIE_PERM_DENY:
      stringKey = "cannot";
      break;
    case MOZ_COOKIE_PERM_FIRST:
      stringKey = "canAccessFirstParty";
      break;
    case MOZ_COOKIE_PERM_SESSION:
      stringKey = "canSession";
      break;
    }
    return this._bundle.getString(stringKey);
  },

  onSelectionChange: function(aEvent) {
    var that         = gCookieKeeperPermissionsXUL;
    var hasSelection = that._tree.view.selection.count > 0;
    var hasRows      = that._tree.view.rowCount > 0;
    document.getElementById("btnRemovePermission").disabled     = !hasRows || !hasSelection;
    document.getElementById("btnRemoveAllPermissions").disabled = !hasRows;
    return false;
  },

  onTextboxURLKeypress: function(aEvent) {
    if (aEvent.keyCode == KeyEvent.DOM_VK_RETURN) {
      let btnBlock = document.getElementById("btnBlock");
      if (!btnBlock.disabled) {
        document.getElementById("btnBlock").click();
        document.getElementById("textBoxUrl").focus();
      }
      aEvent.preventDefault();
      return false;
    }

    return true;
  },

  onTextboxURLInput: function(aEvent) {
    let disabled = this.value.length == 0;
    document.getElementById("btnSession").disabled = disabled;
    document.getElementById("btnBlock").disabled   = disabled;
    document.getElementById("btnAllow").disabled   = disabled;
    return false;
  },

  onAddPermission: function(aEvent) {
    let textBoxUrl = document.getElementById("textBoxUrl");
    let that       = gCookieKeeperPermissionsXUL;
    let oPManager  = that._oPermManager;
    let capability = oPManager.PERM_DEFAULT;
    let value      = textBoxUrl.value.trim();
    let uri        = null;

    if (value.length == 0) {
      log.d("Input value is empty");
      return false;
    }

    if (this.id == "btnBlock") {
      capability = oPManager.PERM_DENY;
    } else if (this.id == "btnAllow") {
      capability = oPManager.PERM_ALLOW;
    } else if (this.id == "btnSession") {
      capability = oPManager.PERM_SESSION;
    }

    try {
      try {
        uri = Services.io.newURI(value, null, null);
      } catch (ex) {
        log.d("missing scheme, trying with http");
        uri = Services.io.newURI("http://" + value, null, null);
      }
    } catch (ex) {
      log.e("Invalid domain: " + value);
      log.e(ex);
      return false;
    }

    try {
      if (oPManager.add(uri, capability) == null) {
        uri = null;
      } else {
        document.getElementById("btnSession").disabled = true;
        document.getElementById("btnBlock").disabled   = true;
        document.getElementById("btnAllow").disabled   = true;
        textBoxUrl.value = "";
        textBoxUrl.focus();
      }
    } catch (ex) {
      log.e(ex);
      uri = null;
    }

    if (uri == null) {
      // FIXME: translation
      window.alert("Invalid URI: '" + value + "'");
    }
    return false;
  },

  onPermissionKeyPress: function(aEvent) {
    if (aEvent.keyCode == aEvent.DOM_VK_DELETE) {
      return gCookieKeeperPermissionsXUL.onDeletePermission(aEvent);
    } else if (aEvent.code == "KeyC" && aEvent.ctrlKey) {
      let sClipboard = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
      let that       = gCookieKeeperPermissionsXUL;
      let selection  = that._tree.view.selection;
      let rc         = selection.getRangeCount();
      let aPerms     = that._oPermManager.aPermissions;
      let cpyText    = "";

      for (let i = 0; i < rc; i++) {
        let min = { }; let max = { };
        selection.getRangeAt(i, min, max);
        for (let j = min.value; j <= max.value; j++) {
          if (cpyText.length > 0) {
            cpyText = cpyText + "\n";
          }
          cpyText = cpyText + aPerms[j].origin;
        }
      }

      sClipboard.copyString(cpyText);
      return false;
    }

    return true;
  },

  onDeletePermission: function(aEvent) {
    var that          = gCookieKeeperPermissionsXUL;
    var selection     = that._tree.view.selection;
    var tbo           = that._tree.treeBoxObject;
    var aDeletedItems = [];
    var aPerms        = that._oPermManager.aPermissions;
    var nextSelection = 0;
    var rc            = selection.getRangeCount();

    for (let i = 0; i < rc; i++) {
      let min = { }; let max = { };
      selection.getRangeAt(i, min, max);
      for (let j = min.value; j <= max.value; j++) {
        aDeletedItems.push(aPerms[j]);
        nextSelection = Math.max(j, nextSelection);
      }
    }

    if (aDeletedItems.length > 0) {
      selection.selectEventsSuppressed = true;
      tbo.beginUpdateBatch();
      that._oPermManager.removePermChangeObserver(that.onPermissionChange);

      for (let i=0; i<aDeletedItems.length; i++) {
        that._oPermManager.remove(aDeletedItems[i]);
      }
      tbo.rowCountChanged(0, -aDeletedItems.length);

      if (aPerms.length > 0) {
        nextSelection = Math.min(aPerms.length - 1, nextSelection);
        selection.select(nextSelection);
        tbo.ensureRowIsVisible(nextSelection);
        that._tree.focus();
      } else {
        document.getElementById("btnRemovePermission").disabled     = true;
        document.getElementById("btnRemoveAllPermissions").disabled = true;
      }

      that._oPermManager.addPermChangeObserver(that.onPermissionChange);
      tbo.endUpdateBatch();
      selection.selectEventsSuppressed = false;

      return false;
    }

    return true;
  },

  onDeleteAllPermissions: function(aEvent) {
    var that   = gCookieKeeperPermissionsXUL;
    var aPerms = that._oPermManager.aPermissions;
    var nRows  = aPerms.length;
    var aTmp   = new Array(nRows);
    var sel    = that._tree.view.selection;

    that._tree.treeBoxObject.beginUpdateBatch();
    sel.clearSelection();

    for (let i=0; i<nRows; i++) {
      aTmp[i] = aPerms[i];
    }

    for (let i=0; i<nRows; i++) {
      that._oPermManager.remove(aTmp[i]);
    }

    that._tree.treeBoxObject.endUpdateBatch();
    document.getElementById("btnRemovePermission").disabled     = true;
    document.getElementById("btnRemoveAllPermissions").disabled = true;

    return false;
  },

  onContextMenu: function(aEvent) {
    var that            = gCookieKeeperPermissionsXUL;
    var seln            = that._view.selection;
    var showContextMenu = that._view.rowCount > 0 && seln.count > 0;
    if (!showContextMenu) {
      aEvent.preventDefault();
      return false;
    }
    return true;
  },

  onShowContextMenu: function(aEvent) {
    var that            = gCookieKeeperPermissionsXUL;
    var aPerms          = that._oPermManager.aPermissions;
    var seln            = that._view.selection;
    var mpccPermAllow   = document.getElementById("mpcc-perm-allow");
    var mpccPermSession = document.getElementById("mpcc-perm-session");
    var mpccPermDeny    = document.getElementById("mpcc-perm-deny");

    that._lastSelectedItems = [];

    if (seln.currentIndex == -1 || seln.count == 0) {
      return false;
    }

    mpccPermAllow.hidden   = false;
    mpccPermSession.hidden = false;
    mpccPermDeny.hidden    = false;

    // Hide unusefull menu items
    if (seln.count > 1) {
      let nPermAllow   = 0;
      let nPermSession = 0;
      let nPermDeny    = 0;
      let nTotal       = 0;
      let rangeCount   = seln.getRangeCount();

      for (let i = 0; i < rangeCount; i++) {
        let min = {}; var max = {};
        seln.getRangeAt(i, min, max);

        for (let j = min.value; j <= max.value; j++) {
          nTotal++;
          let item = aPerms[j];
          that._lastSelectedItems.push(item);

          switch (item.capability) {
          case MOZ_COOKIE_PERM_ALLOW:
            nPermAllow++;
            break;
          case MOZ_COOKIE_PERM_SESSION:
            nPermSession++;
            break;
          case MOZ_COOKIE_PERM_DENY:
            nPermDeny++;
            break;
          }
        }
      }

      if (nTotal == nPermAllow) {
        mpccPermAllow.hidden = true;
      } else if (nTotal == nPermSession) {
        mpccPermSession.hidden = true;
      } else if (nTotal == nPermDeny) {
        mpccPermDeny.hidden = true;
      }

    } else {
      let item = aPerms[seln.currentIndex];
      that._lastSelectedItems.push(item);

      switch (item.capability) {
      case MOZ_COOKIE_PERM_ALLOW:
        mpccPermAllow.hidden = true;
        break;
      case MOZ_COOKIE_PERM_SESSION:
        mpccPermSession.hidden = true;
        break;
      case MOZ_COOKIE_PERM_DENY:
        mpccPermDeny.hidden = true;
        break;
      }
    }

    return true;
  },

  onContextMenuItem: function(aEvent) {
    let that = gCookieKeeperPermissionsXUL;

    if (this.id == "mpcc-delete") {
      that.onDeletePermission();

    } else if (this.id == "mpcc-copy-domain") {
      let sClipboard = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
      let cpyText = "";

      for (let i=0; i<that._lastSelectedItems.length; i++) {
        let oCKPerm = that._lastSelectedItems[i];
        if (cpyText.length > 0) {
          cpyText = cpyText + "\n";
        }
        cpyText = cpyText + oCKPerm.origin;
      }
      sClipboard.copyString(cpyText);

    } else {
      let capability = that._oPermManager.PERM_DEFAULT;

      if (this.id == "mpcc-perm-allow") {
        capability = that._oPermManager.PERM_ALLOW;
      } else if (this.id == "mpcc-perm-session") {
        capability = that._oPermManager.PERM_SESSION;
      } else if (this.id == "mpcc-perm-deny") {
        capability = that._oPermManager.PERM_DENY;
      } else {
        log.e("Unknown context menu action: ", this, aEvent);
        that._lastSelectedItems = null;
        return false;
      }

      for (let i=0; i<that._lastSelectedItems.length; i++) {
        let oCKPerm = that._lastSelectedItems[i];
        oCKPerm.capability = capability;
      }
    }

    that._lastSelectedItems = null;

    return false;
  },

  onPermissionSort: function(aEvent) {
    var that        = gCookieKeeperPermissionsXUL;
    var locale      = "en";
    var sortColumn  = 0;
    var aSortColumn = 0;

    function pcompare(a, b) {
      let cmp = 0;

      if (aSortColumn == 2) {
        let ca = that.getCapabilityString(a.capability);
        let cb = that.getCapabilityString(b.capability);
        cmp = ca.localeCompare(cb, locale);
      }
      if (cmp == 0) {
        cmp = a.compare(b);
      }

      return sortColumn > 0 ? cmp : -cmp;
    }

    if (that._oPermManager.aPermissions.length == 0) {
      return false;
    }

    if (aEvent == null) {
      sortColumn = that._sortColumn;
    } else {
      sortColumn = (this.id == "treeColSite") ? 1 : 2;
      if (Math.abs(that._sortColumn) == Math.abs(sortColumn)) {
        sortColumn = -that._sortColumn;
      }
      that._sortColumn = sortColumn;
    }

    aSortColumn = Math.abs(sortColumn);
    if (aSortColumn == 2) {
      let chrome = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULChromeRegistry);
      locale = chrome.getSelectedLocale("global");
    }
    that._oPermManager.aPermissions.sort(pcompare);

    let tbo = that._tree.treeBoxObject;
    let sel = that._tree.view.selection;
    sel.clearSelection();
    sel.select(0);
    tbo.invalidate();
    tbo.ensureRowIsVisible(0);

    if (Math.abs(sortColumn) == 1) {
      let elem = document.getElementById("treeColSite");
      elem.setAttribute("sortDirection", sortColumn > 0 ? "ascending" : "descending");
      elem = document.getElementById("treeColCapability");
      elem.removeAttribute("sortDirection");
    } else {
      let elem = document.getElementById("treeColSite");
      elem.removeAttribute("sortDirection");
      elem = document.getElementById("treeColCapability");
      elem.setAttribute("sortDirection", sortColumn > 0 ? "ascending" : "descending");
    }

    return false;
  },

  onPermissionChange: function(oCKPerm, sChange) {
    var that = gCookieKeeperPermissionsXUL;

    if (sChange == "added") {
      that._tree.treeBoxObject.rowCountChanged(that._view.rowCount - 1, 1);
      that.onPermissionSort(null);

    } else if (sChange == "deleted") {
      let rowCount = that._view.rowCount;
      that._tree.treeBoxObject.rowCountChanged(rowCount + 1, -1);

      if (rowCount == 0) {
        document.getElementById("btnRemovePermission").disabled     = true;
        document.getElementById("btnRemoveAllPermissions").disabled = true;
      } else {
        let tbo = that._tree.treeBoxObject;
        let sel = that._tree.view.selection;
        sel.clearSelection();
        sel.select(0);
        tbo.ensureRowIsVisible(0);
        document.getElementById("btnRemovePermission").disabled     = false;
        document.getElementById("btnRemoveAllPermissions").disabled = false;
      }

    } else if (sChange == "changed") {
      that._tree.treeBoxObject.invalidate();
    } else {
      log.e("Unknown changed type:", sChange);
    }
  }
};

var gCookieKeeperPermissionsHTML = {
  _oDynTable   : null,
  _oPermManager: null,
  _eventLock   : false,

  init: function() {
    try {
      this.uninit();

      let hTable     = document.getElementById("perm_table");
      let hTBody     = document.getElementById("perm_tbody");
      let hBtnRem    = document.getElementById("perm_button_remove");
      let hBtnRemAll = document.getElementById("perm_button_remove_all");

      this._oDynTable    = new DynTable(hTable, "perm_cookies");
      this._oPermManager = new CKPermissionsManager();

      this._oPermManager.addPermChangeObserver(this.onPermChange);

      let aPerms = this._oPermManager.fetchCookiesPermissions();
      for (let i=0; i<aPerms.length; i++) {
        let oCKPerm = aPerms[i];
        let hTrElem = this.createTrElement(oCKPerm);
        hTBody.appendChild(hTrElem);
      }

      // Allow sorting
      this._oDynTable.enableSortColumn(this._compare);
      // Do initial sorting
      this._oDynTable.doSort(null);
      // Initial column resize
      this._oDynTable.resizeBodyColumns();
      // Allow selection
      this._oDynTable.setSelectListener(this.onSelectionChange);

      // Allow to select the table
      hTable.setAttribute("tabindex", "2");

      // Enable "Remove All" button
      if (aPerms.length > 0) {
        hBtnRemAll.removeAttribute("disabled");
      } else {
        hBtnRemAll.setAttribute("disabled", "disabled");
      }

      hBtnRem.addEventListener("click", this.onRemove, false);
      hBtnRemAll.addEventListener("click", this.onRemoveAll, false);
      document.getElementById("perm_input_address").focus();
    } catch (ex) {
      log.e(ex);
    }
  },

  uninit: function() {
    let hInput     = document.getElementById("perm_input_address");
    let hTable     = document.getElementById("perm_table");
    let hTBody     = document.getElementById("perm_tbody");
    let hBtnRem    = document.getElementById("perm_button_remove");
    let hBtnRemAll = document.getElementById("perm_button_remove_all");

    hInput.value = "";
    this.onInputChange(null);

    if (this._oDynTable != null) {
      this._oDynTable.destructor();
      this._oDynTable = null;
    }
    if (this._oPermManager != null) {
      this._oPermManager.destructor();
      this._oPermManager = null;
    }

    hTable.removeAttribute("tabindex");

    // Remove tbody child elements
    while (hTBody.firstElementChild) {
      let hTrElem = hTBody.firstElementChild;
      delete hTrElem.oCKPerm;
      hTBody.removeChild(hTrElem);
    }

    // Disabled buttons
    hBtnRem.setAttribute("disabled", "disabled");
    hBtnRem.removeEventListener("click", this.onRemove, false);
    hBtnRemAll.setAttribute("disabled", "disabled");
    hBtnRemAll.removeEventListener("click", this.onRemoveAll, false);

    this._eventLock = false;
  },

  translatePerm: function(oCKPerm) {
    let trId;
    switch (oCKPerm.capability) {
    case MOZ_COOKIE_PERM_ALLOW:
      trId = "perms.cookies.allow";
      break;
    case MOZ_COOKIE_PERM_DENY:
      trId = "perms.cookies.block";
      break;
    case MOZ_COOKIE_PERM_FIRST:
      trId = "perms.cookies.canAccessFirstParty";
      break;
    case MOZ_COOKIE_PERM_SESSION:
      trId = "perms.cookies.allow_session";
      break;
    default:
      trId = "perms.cookies.unknown";
    }
    let tr    = gChrome.i18n.getMessage(trId, []);
    return tr == "" ? trId : tr;
  },

  createTrElement: function(oCKPerm) {
    let hTrElem = document.createElement("tr");
    hTrElem.classList.add("table_scroll_contentrow");
    hTrElem.setAttribute("contextmenu", "perm_menu_context");
    hTrElem.oCKPerm = oCKPerm;

    let hTdElem = document.createElement("td");
    hTdElem.classList.add("table_scroll_contentcell");
    hTdElem.innerHTML = oCKPerm.origin;
    hTrElem.appendChild(hTdElem);

    hTdElem = document.createElement("td");
    hTdElem.classList.add("table_scroll_contentcell");
    hTdElem.innerHTML = this.translatePerm(oCKPerm);
    hTrElem.appendChild(hTdElem);

    return hTrElem;
  },

  onPermChange: function(oCKPerm, sChange) {
    let that = gCookieKeeperPermissionsHTML;
    if (that._eventLock) {
      return;
    }

    let hTBody    = document.getElementById("perm_tbody");
    let btnRemAll = document.getElementById("perm_button_remove_all");

    if (sChange == "added") {
      let hTrElem = that.createTrElement(oCKPerm);
      hTBody.appendChild(hTrElem);
      that._oDynTable.resizeBodyColumns();
    } else if (sChange == "changed") {
      let nElems  = hTBody.children.length;
      for (let i=0; i<nElems; i++) {
        let hTrBodyElem = hTBody.children[i];
        if (hTrBodyElem.oCKPerm.same(oCKPerm)) {
          // Update the permission text value
          hTrBodyElem.children[1].innerHTML = that.translatePerm(oCKPerm);
        }
      }
    } else if (sChange == "deleted") {
      let nIdxLastSelected = that._oDynTable.nIdxLastSelected;
      let nElems           = hTBody.children.length;
      let nIdxDel          = -1;

      // Find the deleted row
      for (let i=0; i<nElems; i++) {
        let hTrBodyElem = hTBody.children[i];
        if (hTrBodyElem.oCKPerm.equals(oCKPerm)) {
          nIdxDel = i;
          nElems--;
          hTBody.removeChild(hTrBodyElem);
          break;
        }
      }

      if (nIdxDel == -1) {
        log.e("Permission for " + oCKPerm.host + " not found");
      } else for (let i=nIdxDel; i<nElems; i++) {
        // Update the styles odd/even
        let hTrBodyElem = hTBody.children[i];
        if ((i % 2) == 0) {
          hTrBodyElem.classList.remove("table_scroll_contentrow_odd");
          hTrBodyElem.classList.add("table_scroll_contentrow_even");
        } else {
          hTrBodyElem.classList.remove("table_scroll_contentrow_even");
          hTrBodyElem.classList.add("table_scroll_contentrow_odd");
        }
      }
      that._oDynTable.updateSelection();

      if (hTBody.children.length == 0) {
        document.getElementById("perm_button_remove").setAttribute("disabled", "disabled");;
        document.getElementById("perm_button_remove_all").setAttribute("disabled", "disabled");;
      } else if (that._oDynTable.nIdxLastSelected == -1 && nIdxLastSelected != -1) {
        nIdxLastSelected = Math.min(nIdxLastSelected, hTBody.children.length - 1);

      }

    }

    if (hTBody.children.length > 0) {
      btnRemAll.removeAttribute("disabled");
    } else {
      btnRemAll.setAttribute("disabled", "disabled");
    }
  },

  onSelectionChange: function(aAddedItems, aRemovedItems, aCurrentItems) {
    let btn = document.getElementById("perm_button_remove");

    if (aCurrentItems.length > 0) {
      btn.removeAttribute("disabled");
    } else {
      btn.setAttribute("disabled", "disabled");
    }
  },

  // Compare 2 rows (<tr> elements) of the <tbody>.
  // Use to sort the table
  _compare: function(a, b) {
    let aSortColumns = gCookieKeeperPermissionsHTML._oDynTable.aSortColumns;
    if (aSortColumns == null || !a.oCKPerm || !b.oCKPerm) {
      return 0;
    }

    let sortNormal = aSortColumns[0] > 0;
    let nCols      = aSortColumns.length;
    let aHTML      = a.children[1].innerHTML.trim();
    let bHTML      = b.children[1].innerHTML.trim();

    for (let i=0; i<nCols; i++) {
      let nColId = Math.abs(aSortColumns[i]);
      let cmpVal = 0;
      switch (nColId) {
      case 1:
        cmpVal = a.oCKPerm.compare(b.oCKPerm);
        break;
      case 2:
        cmpVal = aHTML.toLowerCase().localeCompare(bHTML.toLowerCase());
        break;
      }

      if (cmpVal != 0) {
        return sortNormal ? cmpVal : -cmpVal;
      }
    }
    return 0;
  },

  onRemove: function(evtClick) {
    let hTBody     = document.getElementById("perm_tbody");
    let hBtnRem    = document.getElementById("perm_button_remove");
    let hBtnRemAll = document.getElementById("perm_button_remove_all");
    let that       = gCookieKeeperPermissionsHTML;
    let aSelected  = that._oDynTable.aSelected;
    let nSelected  = aSelected.length;
    let aHtrElems  = new Array(nSelected);

    if (nSelected == 0) {
      return false;
    }

    that._eventLock = true;
    that._oDynTable.resetSelection();

    for (let i=0; i<nSelected; i++) {
      aHtrElems[i] = hTBody.children[aSelected[i]];
    }
    for (let i=0; i<nSelected; i++) {
      let hTrElem = aHtrElems[i];
      that._oPermManager.remove(hTrElem.oCKPerm);
      hTBody.removeChild(hTrElem);
    }
    hBtnRem.setAttribute("disabled", "disabled");
    if (hTBody.children.length == 0) {
      hBtnRemAll.setAttribute("disabled", "disabled");
    } else for (let i=0; i<hTBody.children.length; i++) {
      // Update style odd/even to rows
      let hTrBodyElem = hTBody.children[i];
      if ((i % 2) == 0) {
        hTrBodyElem.classList.remove("table_scroll_contentrow_odd");
        hTrBodyElem.classList.add("table_scroll_contentrow_even");
      } else {
        hTrBodyElem.classList.remove("table_scroll_contentrow_even");
        hTrBodyElem.classList.add("table_scroll_contentrow_odd");
      }
    }

    that._eventLock = false;
    return false;
  },

  onRemoveAll: function(evtClick) {
    let that       = gCookieKeeperPermissionsHTML;
    let hTBody     = document.getElementById("perm_tbody");
    let hBtnRem    = document.getElementById("perm_button_remove");
    let hBtnRemAll = document.getElementById("perm_button_remove_all");
    let oPManager  = that._oPermManager;
    let nElems     = hTBody.children.length;
    let aPerms     = new Array(nElems);

    that._eventLock = true;
    that._oDynTable.resetSelection();

    for (let i=0; i<nElems; i++) {
      let hTrBodyElem = hTBody.firstElementChild;
      aPerms[i] = hTrBodyElem.oCKPerm;
      hTBody.removeChild(hTrBodyElem);
    }

    for (let i=0; i<nElems; i++) {
      let oCKPerm = aPerms[i];
      if (!oPManager.remove(oCKPerm)) {
        log.e("Error removing permission of " + oCKPerm.host);
      }
    }

    hBtnRem.setAttribute("disabled", "disabled");
    hBtnRemAll.setAttribute("disabled", "disabled");

    that._eventLock = false;
  },

  onInputChange: function(evtChange) {
    let hInput      = document.getElementById("perm_input_address");
    let hBtnBlock   = document.getElementById("perm_button_block");
    let hBtnAllow   = document.getElementById("perm_button_allow");
    let hBtnSession = document.getElementById("perm_button_session");

    if (hInput.value.length > 0) {
      hBtnBlock.removeAttribute("disabled");
      hBtnBlock.addEventListener("click", this.onAddPerm, false);

      hBtnAllow.removeAttribute("disabled");
      hBtnAllow.addEventListener("click", this.onAddPerm, false);

      hBtnSession.removeAttribute("disabled");
      hBtnSession.addEventListener("click", this.onAddPerm, false);
    } else {
      hBtnBlock.setAttribute("disabled", "disabled");
      hBtnBlock.removeEventListener("click", this.onAddPerm, false);

      hBtnAllow.setAttribute("disabled", "disabled");
      hBtnAllow.removeEventListener("click", this.onAddPerm, false);

      hBtnSession.setAttribute("disabled", "disabled");
      hBtnSession.removeEventListener("click", this.onAddPerm, false);
    }

    return true;
  },

  onAddPerm: function(evtClick) {
    let hInput     = document.getElementById("perm_input_address");
    let that       = gCookieKeeperPermissionsHTML;
    let oPManager  = that._oPermManager;
    let capability = oPManager.PERM_DEFAULT;
    let value      = hInput.value.trim();
    let uri        = null;

    if (value.length == 0) {
      window.alert("Empty URI");
      hInput.value = "";
      that.onInputChange(null);
      return true;
    }

    if (this.id == "perm_button_block") {
      capability = oPManager.PERM_DENY;
    } else if (this.id == "perm_button_allow") {
      capability = oPManager.PERM_ALLOW;
    } else if (this.id == "perm_button_session") {
      capability = oPManager.PERM_SESSION;
    }

    try {
      try {
        uri = Services.io.newURI(value, null, null);
      } catch (ex) {
        uri = Services.io.newURI("http://" + value, null, null);
      }
    } catch (ex) {
      log.e("Invalid domain: " + value);
      log.e(ex);
      return true;
    }


    try {
      if (oPManager.add(uri, capability) == null) {
        uri = null;
      } else {
        hInput.value = "";
        that.onInputChange(null);
        hInput.focus();
      }
    } catch (ex) {
      log.e(ex);
      uri = null;
    }

    if (uri == null) {
      // FIXME: translation
      window.alert("Invalid URI: '" + value + "'");
    }

    return true;
  },

  onContextMenu: function(evtContextMenu) {
    let hTBody    = document.getElementById("perm_tbody");
    let item      = evtContextMenu.target.id;
    let aSelected = this._oDynTable.aSelected;

    if (item == "perm_menuitem_remove") {
      this.onRemove(evtContextMenu);
    } else if (item == "perm_menuitem_copy") {
      let sClipboard = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
      if (aSelected.length > 1) {
        let cpyText = "";
        for (let i=0; i<aSelected.length; i++) {
          let hTrElem = hTBody.children[aSelected[i]];
          cpyText += hTrElem.oCKPerm.host + "\n";
        }
        sClipboard.copyString(cpyText);
      } else if (this._oDynTable.nIdxLastSelected != -1) {
        let hTrElem = hTBody.children[this._oDynTable.nIdxLastSelected];
        sClipboard.copyString(hTrElem.oCKPerm.host);
      }

    } else {
      // Change permission value
      let capability = this._oPermManager.PERM_DEFAULT;

      if (item == "perm_menuitem_allow") {
        capability = this._oPermManager.PERM_ALLOW;
      } else if (item == "perm_menuitem_block") {
        capability = this._oPermManager.PERM_DENY;
      } else if (item == "perm_menuitem_session") {
        capability = this._oPermManager.PERM_SESSION;
      }

      for (let i=0; i<aSelected.length; i++) {
        let hTrElem = hTBody.children[aSelected[i]];
        hTrElem.oCKPerm.capability = capability;
      }
    }

    return false;
  }
};
