/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Locale (l10n) management from string bundle.
 *
 * ************************************************************************
 */

"use strict";

const DEFAULT_LOCALE = "en";

// Localization instance:
var l10n = null;

function Localization(_addonURI, _filename) {
  function getBundle(_locale) {
    let e = null;
    let p = _addonURI + "chrome/locale/" + _locale + "/" + _filename;
    let b = Services.strings.createBundle(p);
    try {
      e = b.getSimpleEnumeration();
      return b;
    } catch (ex) {
      log.w("Bundle file '" + _filename + "' for '" + _locale + "' not found");
    }
    return null;
  }

  function _tr(textId) {
    let text = null;

    if (typeof(textId) == "string") {
      try {
        text = this.bundle.GetStringFromName(textId);
      } catch (ex) {
        log.e("" + textId + " not found in " + this.filename);
        Cu.reportError(ex);
      }

      if (text == null) {
        if (this.dBundle == null) {
          this.dBundle  = this._getBundle(DEFAULT_LOCALE);
        }
        try {
          text = this.dBundle.GetStringFromName(textId);
        } catch (ex) {
          log.e("" + textId + " not found in " + this.filename + " for default (\"en\") language");
          Cu.reportError(ex);
        }
      }
    }

    if (text == null) {
      text = "" + textId;
    }

    return text;
  }

  try {

    this.addonURI = _addonURI;
    this.filename = _filename;
    this.dBundle  = null;
    this.locale   = DEFAULT_LOCALE;
    this.bundle   = null;
    let chrome    = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULChromeRegistry);
    let gLocale   = chrome.getSelectedLocale("global");
    let _bundle   = getBundle(gLocale);

    if (_bundle == null && gLocale.indexOf("-") != -1) {
      gLocale = gLocale.split("-")[0];
      _bundle = getBundle(gLocale);
    }
    if (_bundle == null) {
      this.dBundle = getBundle(DEFAULT_LOCALE);
      gLocale = DEFAULT_LOCALE;
      _bundle = this.dBundle;
      log.i("Using default locale " + DEFAULT_LOCALE);
    }
    this.locale   = gLocale;
    this.bundle   = _bundle;
    this.tr       = _tr;

  } catch (ex) {
    Cu.reportError(ex);
  }
}
